/*-----------------------------------------------------------------------------------------
   This software is in the public domain, furnished "as is", without technical
   support, and with no warranty, express or implied, as to its usefulness for
   any purpose.

   Yet Another Multi Switch - "YAMS" Decoder
   The sketch reeds servo-signals from two channels with encoded signals from two YAMS encoder

   With Jumper open only the first channel will be used

   Author: Dirk Wiedemann
   Date: 2018/02/09

   Version 0.2
*/

#include <ServoIn.h>
#include <Timer1.h>


#define SERVOS 2

uint16_t g_values[SERVOS];                    // output buffer for ServoIn
uint8_t  g_workIn[SERVOIN_WORK_SIZE(SERVOS)]; // we need to have a work buffer for the ServoIn class

int swPin1_1 = 2;
int swPin1_2 = 3;
int swPin1_3 = 4;
int swPin1_4 = 5;
int swPin1_5 = 6;
int swPin2_1 = 7;
int swPin2_2 = 10;
int swPin2_3 = 11;
int swPin2_4 = 12;
int swPin2_5 = 13;
int jpPin = 14;          // jumper to GND, open to enable the second channel


int hyst_low_1 = 0;
int hyst_high_1 = 0;
int swStatus_1 = 0;
int swOld_1 = 0;
int hyst_low_2 = 0;
int hyst_high_2 = 0;
int swStatus_2 = 0;
int swOld_2 = 0;
int jumperState = 0;
int temp = 0;

// Thresholds to separate every switch state from his neighbors, could be customized for individual encoders

int threshold_1[33] = { 800 , 1002 , 1028 , 1058 , 1091 , 1123 , 1154 , 1184 , 1218 , 1253 , 1283 , 1314 , 1347 ,
                        1379 , 1409 , 1440 , 1478 , 1518 , 1552 , 1586 , 1621 , 1657 , 1691 , 1725 , 1762 , 1799 ,
                        1833 , 1867 , 1903 , 1939 , 1972 , 2002 , 2200
                      };   // first encoder

int threshold_2[33] = { 800 , 1002 , 1028 , 1058 , 1091 , 1123 , 1154 , 1184 , 1218 , 1253 , 1283 , 1314 , 1347 ,
                        1379 , 1409 , 1440 , 1478 , 1518 , 1552 , 1586 , 1621 , 1657 , 1691 , 1725 , 1762 , 1799 ,
                        1833 , 1867 , 1903 , 1939 , 1972 , 2002 , 2200
                      };   // second encoder

rc::ServoIn g_ServoIn(g_values, g_workIn, SERVOS);


void setup()
{
  // Initialize timer1, this is required for all features that use Timer1
  // (PPMIn/PPMOut/ServoIn/ServoOut)
  rc::Timer1::init();

  // We use pin 8-9 as Servo input pins
  pinMode(8, INPUT);
  pinMode(9, INPUT);


  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);

  pinMode(14, INPUT_PULLUP);

  Serial.begin(9600);

  // We use pin change interrupts to detect changes in the signal
  // If you're unfamiliar with how this works, please look up some
  // article or tutorial on the subject.

  // only allow pin change interrupts for PB0-1 (digital pins 8-9)
  PCMSK0 = (1 << PCINT0) | (1 << PCINT1);

  // enable pin change interrupt 0
  PCICR = (1 << PCIE0);

  // start listening
  g_ServoIn.start();

}

void loop()
{
  // update incoming values
  g_ServoIn.update();

  jumperState = digitalRead(jpPin);

  // ************************
  // Processing first channel
  // ************************

  for (int i = 0; i <= 31; i++) {
    if (g_values[0] > threshold_1[i] && g_values[0] <= threshold_1[i + 1]) {
      swStatus_1 = i;
    }
  }

  // hysteresis for flickering lowest stati

  if (swStatus_1 < 2 && swOld_1 != swStatus_1) {
    if (hyst_low_1 >= 5) {
      hyst_low_1 = 0;
    }
    else {
      hyst_low_1 = hyst_low_1 + 1;
      swStatus_1 = swOld_1;
    }
  }

  // hysteresis for flickering highest stati

  if (swStatus_1 > 29 && swOld_1 != swStatus_1) {
    if (hyst_high_1 >= 5) {
      hyst_high_1 = 0;
    }
    else {
      hyst_high_1 = hyst_high_1 + 1;
      swStatus_1 = swOld_1;
    }
  }

  // Setting old swStatus_1

  swOld_1 = swStatus_1;

  // Setting output pins

  temp = swStatus_1;
  if (temp >= 16) {
    digitalWrite(swPin1_1, HIGH);
    temp = temp - 16;
  }
  else {
    digitalWrite(swPin1_1, LOW);
  }

  if (temp >= 8) {
    digitalWrite(swPin1_2, HIGH);
    temp = temp - 8;
  }
  else {
    digitalWrite(swPin1_2, LOW);
  }

  if (temp >= 4) {
    digitalWrite(swPin1_3, HIGH);
    temp = temp - 4;
  }
  else {
    digitalWrite(swPin1_3, LOW);
  }

  if (temp >= 2) {
    digitalWrite(swPin1_4, HIGH);
    temp = temp - 2;
  }
  else {
    digitalWrite(swPin1_4, LOW);
  }

  if (temp >= 1) {
    digitalWrite(swPin1_5, HIGH);
    temp = temp - 1;
  }
  else {
    digitalWrite(swPin1_5, LOW);
  }

  // *************************
  // Processing second channel
  // *************************

  if (jumperState == LOW)         // if jumper is open skip second channel an set all ports to low
  {
    for (int i = 0; i <= 31; i++) {
      if (g_values[1] > threshold_2[i] && g_values[1] <= threshold_2[i + 1]) {
        swStatus_2 = i;
      }
    }

    // hysteresis for flickering lowest stati

    if (swStatus_2 < 2 && swOld_2 != swStatus_2) {
      if (hyst_low_2 >= 5) {
        hyst_low_2 = 0;
      }
      else {
        hyst_low_2 = hyst_low_2 + 1;
        swStatus_2 = swOld_2;
      }
    }

    // hysteresis for flickering highest stati

    if (swStatus_2 > 29 && swOld_2 != swStatus_2) {
      if (hyst_high_2 >= 5) {
        hyst_high_2 = 0;
      }
      else {
        hyst_high_2 = hyst_high_2 + 1;
        swStatus_2 = swOld_2;
      }
    }

    // Setting old swStatus

    swOld_2 = swStatus_2;

    // Setting output pins

    temp = swStatus_2;
    if (temp >= 16) {
      digitalWrite(swPin2_1, HIGH);
      temp = temp - 16;
    }
    else {
      digitalWrite(swPin2_1, LOW);
    }

    if (temp >= 8) {
      digitalWrite(swPin2_2, HIGH);
      temp = temp - 8;
    }
    else {
      digitalWrite(swPin2_2, LOW);
    }

    if (temp >= 4) {
      digitalWrite(swPin2_3, HIGH);
      temp = temp - 4;
    }
    else {
      digitalWrite(swPin2_3, LOW);
    }

    if (temp >= 2) {
      digitalWrite(swPin2_4, HIGH);
      temp = temp - 2;
    }
    else {
      digitalWrite(swPin2_4, LOW);
    }

    if (temp >= 1) {
      digitalWrite(swPin2_5, HIGH);
      temp = temp - 1;
    }
    else {
      digitalWrite(swPin2_5, LOW);
    }
  }
  else {
    digitalWrite(swPin2_1, LOW);
    digitalWrite(swPin2_2, LOW);
    digitalWrite(swPin2_3, LOW);
    digitalWrite(swPin2_4, LOW);
    digitalWrite(swPin2_5, LOW);
  }


  // ******************************
  // Terminal output
  // ******************************

  Serial.write("\n Channel 1: ");
  Serial.print(g_values[0]);
  Serial.write("            Channel 2: ");
  if (jumperState == LOW) {
    Serial.print(g_values[1]);
  }
  else {
    Serial.write("not used");
  }
}


// Interrupt handling code below, this needs cleaning

static uint8_t lastB = 0;

// Pin change port 0 interrupt
ISR(PCINT0_vect)
{
  // we need to call the ServoIn ISR here, keep code in the ISR to a minimum!
  uint8_t newB = PINB;
  uint8_t chgB = newB ^ lastB; // bitwise XOR will set all bits that have changed
  lastB = newB;

  // has any of the pins changed?
  if (chgB)
  {
    // find out which pin has changed
    if (chgB & _BV(0))
    {
      g_ServoIn.pinChanged(0, newB & _BV(0));
    }
    if (chgB & _BV(1))
    {
      g_ServoIn.pinChanged(1, newB & _BV(1));
    }
  }
}


